-- myConv.e
-- v1.0
-- 2010-11-13
-- conversion functions

-- function dec2str(atom n, atom d)
-- function base2str(atom n, atom b, atom d)
-- function num2str(atom n, atom b, atom d)
-- function str2num(sequence s, atom b)
-- function byte2word(atom l, atom h)
-- function highByte(atom x)
-- function lowByte(atom x)
-- function val(sequence str)
--   returns numerical value of string str
--   decimal separator may be a comma
--   displays a warning on error
-- function atom2string(atom val, sequence fmt)
--   converts numerical value val according to format fmt
--   replaces decimal separator by a comma (for non english speaking countries)
-- function object2string(object s)
--   converts an object in a sequence
--   sub-sequences appear between curly brackets as with print()
--   sub-sequence items are separated by commas as with print()
-- function utf8_to_ascii(sequence s)
--   created from Tommy's Unicode Library 1.2
--   copyright (c) 2003-2004 Tommy Carlier
-- function hexString(sequence s)
--   returns hexadecimal representation of a sequence
-- function dos2unix(sequence s)
--   converts DOS end of lines to Unix ones

include myTypes.e
include myDebug.e

-------------------------------------------------------------------------------

global function dec2str(atom n, atom d)
  sequence result

  result = repeat(32, d)
  for i = d to 1 by -1 do
    result[i] = remainder(n, 10) + 48
    n = floor(n / 10)
  end for
  return result
end function

-------------------------------------------------------------------------------

global function base2str(atom n, atom b, atom d)
  atom c
  sequence result

  result = repeat(32, d)
  c = floor((log(b)/log(2))+0.5)
  for i = d to 1 by -1 do
    result[i] = DIGITS[and_bits(n / power(2, (d-i)*c), b-1)+1]
  end for
  return result
end function

-------------------------------------------------------------------------------

global function num2str(atom n, atom b, atom d)

  if b = 2 then
    if d>32 then d = 32 end if
    return base2str(n, 2, d)
  elsif b = 8 then
    if d>11 then d = 11 end if
    return base2str(n, 8, d)
  elsif b = 10 then
    return dec2str(n, d)
  elsif b = 16 then
    if d>8 then d = 8 end if
    return base2str(n, 16, d)
  else
    return {}
  end if
end function

-------------------------------------------------------------------------------

global function str2num(sequence s, atom b)
  atom l

  l = 0
  for i = 1 to length(s) do
    l = l + (match({s[i]}, DIGITS)-1) * floor(power(b, length(s)-i))
  end for
  return l
end function

-------------------------------------------------------------------------------

global function byte2word(atom l, atom h)
  return h*#100+l
end function

-------------------------------------------------------------------------------

global function highByte(atom x)
  return floor(x / #100)
end function

-------------------------------------------------------------------------------

global function lowByte(atom x)
  return remainder(x, #100)
end function

------------------------------------------------------------------------------

global function val(sequence str)
-- returns numerical value of string str
-- decimal separator may be a comma
-- displays a warning on error
  integer n
  sequence s

  if length(str) = 0 then
    return 0
  end if
  n = find(',', str)
  if n>0 then str[n] = '.' end if
  s = value(str)
  if s[1] = GET_SUCCESS then
    return s[2]
  else
    warnError(sprintf("Erreur de conversion: %s", {str}), 1)
  end if
end function

------------------------------------------------------------------------------

global function atom2string(atom val, sequence fmt)
-- converts numerical value val according to format fmt
-- replaces decimal separator by a comma (for non english speaking countries)
  integer n
  sequence s

  s = sprintf(fmt, val)
  n = find('.', s)
  if n>0 then s[n] = ',' end if
  return s
end function

------------------------------------------------------------------------------

global function object2string(object s)
-- converts an object in a sequence
-- sub-sequences appear between curly brackets as with print()
-- sub-sequence items are separated by commas as with print()
  sequence res

  if integer(s) then
    res = sprintf("%d", s)
  elsif atom(s) then
    if (s >= 0) and (s = floor(s)) then
      res = sprintf("%.0f", s)
    elsif (s < 0) and (s = floor(s+1)) then
      res = sprintf("%.0f", s)
    else
      res = sprintf("%f", s)
    end if
  elsif string(s) then
    res = s
  else
    res = {123}  -- {
    for i=1 to length(s) do
      res &= object2string(s[i])&","
    end for
    res[$] = 125   -- }
  end if
  return res
end function

------------------------------------------------------------------------------

global function utf8_to_ascii(sequence s)
-- created from Tommy's Unicode Library 1.2
-- copyright (c) 2003-2004 Tommy Carlier
  integer i, char, byte, bytes
  sequence res

  i = 1
  res = ""
  while i < length(s) do
    byte = s[i]
    char = 0
    if and_bits(byte,#80) = 0 then bytes = 1
    elsif and_bits(byte,#20) = 0 then bytes = 2
    elsif and_bits(byte,#10) = 0 then bytes = 3
    elsif and_bits(byte,#8) = 0 then bytes = 4
    elsif and_bits(byte,#4) = 0 then bytes = 5
    elsif and_bits(byte,#2) = 0 then bytes = 6
    else return "Incorrect UTF-8 char"
    end if
    byte = and_bits(byte, power(2, 8 - bytes) - 1)
    char += byte * power(64, bytes - 1)
    for n=1 to bytes-1 do
      i += 1
      byte = s[i]
      if byte=0 then
        return "Unexpected end of UTF-8 string"
      elsif byte<0 then
        return "Incorrect UTF-8 char"
      end if
	  byte = and_bits(byte,63)
      char += byte * power(64,bytes-1-n)
    end for
    res &= char
    i += 1
  end while
  return res
end function

------------------------------------------------------------------------------

global function hexString(sequence s)
-- returns hexadecimal representation of a sequence
  sequence result

  result = ""
  for i = 1 to length(s) do
    result &= sprintf("%02x", s[i])
  end for
  return result
end function

------------------------------------------------------------------------------

global function dos2unix(sequence s)
-- converts DOS end of lines to Unix ones
  sequence t
  integer l

  t = {}
  l = match({13,10}, s)
  while l > 0 do
    t = t & s[1..l-1]
    s = s[l+1..length(s)]
    l = match({13,10}, s)
  end while
  t = t & s
  return t
end function


